.\" $Id: tp4d.man.in,v 1.7 2001/04/02 03:38:55 till Exp $
.TH TP4D 8 "Release tp4utils_1_2_beta2"
.SH NAME
tp4d \- a user mode driver for IBM trackpoint (R) devices
.SH SYNOPSIS
.B tp4d
[
.B \-dhic
]
[
.B \-z | \-Z | \-M
.I mode
]
[
.B \-b | \-B | \-2
.I mode2
]
[
.BI \-S " x_y_sens"
]
[
.BI \-V " upper_plateau_velocity"
]
[
.BI \-P " press_sens"
]
[
.BI \-R " release_sens"
]
[
.BI \-H " drag_hysteresis"
]
[
.BI \-D " min_drag_distance"
]
.\"   [
.\"   .BI \-U " backup_range"
.\"   ]
[
.BI \-T " z_avg_timeconst"
]
[
.BI \-J " Jenks_curvature"
]
[
.BI \-m " button_map"
]
[
.BI \-ppriority
]
.SH DESCRIPTION
.B tp4d
is a daemon / user mode driver for IBM trackpoint (R) devices that
are attached to a PC compatible's PS2 auxiliary port. These
trackpoints are common pointing devices in notebook computers such
as the IBM ThinkPad series.
.PP
By default, the trackpoint behaves like an ordinary 2/3 button
mouse. However, these devices exhibit a number of extra features,
e.g. z-axis (vertical pressure) to button-press mapping.
.B tp4d
is designed to make these extra features available. The
.B tp4d
program is a daemon intended to be run in the background. It handles
trackpoint `configuration requests' read from a named pipe
and APM events (since the trackpoint configuration suffers a
`power-on reset' cycle when resuming operation after an APM
suspend/standby request).
.B tp4d
itself may also be used in `interactive mode', i.e. as a client
of a master daemon for issuing configuration requests from the
command line. Together with
.B tp4d
comes the comfortable
.B xtp4
program, a client which may be run under 
.BR X (1).

.SH OPTIONS
Note that at most one of the \fB\-z\fP, \fB\-Z\fP, \fB\-M\fP
or the \fB\-b\fP, \fB\-B\fP, \fB\-2\fP
options may be given. The \fB\-p\fP option is special: no space
must be present between the `p' character and the option argument.
.PP
It is advisable to use the
.B xtp4
utility for experimenting with the different parameter settings.
Once a satisfying setting has been found, the corresponding
command line options may then be supplied (as initial values)
when starting
.BR tp4d .
.IP "\fB\-d"
Run in `debug' mode. Messages are printed to `stderr' instead of
sending them to the system logger. The program keeps running in
the foreground instead of detaching itself from the controlling
terminal and becoming a daemon.
.IP "\fB\-h"
Print short usage information and exit.
.IP "\fB\-i"
Run in `interactive' or `client' mode. In interactive mode,
.B tp4d
tries to contact the server daemon (another instance of 
.B tp4d
which must be up and running) and forwards the settings passed
as command line options along to the server which takes care
of configuring the hardware. If no running server is found, 
.B tp4d
talks to the trackpoint hardware directly and exits. Note that
the settings wont survive an APM suspend/resume cycle in this
case.
.IP "\fB\-z"
Switch off / disable the z-axis. None of the stick-press -> mouse
button event mappings will work (and using
.B tp4d
is maybe pointless).
.IP "\fB\-Z"
Enable z-axis. Make the z-axis related features available.
.IP "\fB\-M\fR\fI mode\fR [0x00..0x7f]"
Set the mode word to \fImode\fP, which must be a number between
0x00 and 0x7f. Every bit of the mode word has a special meaning.
.PP
.RS
.IP "0x01 z-axis enable [1] / disable [0].
.IP "0x02 reserved (consult IBM docs)"
.IP "0x04 reserved (consult IBM docs)"
.IP "0x08 swap left/right"
Pressing the stick to the right will generate `move-left' events.
.sp
.IP "0x10 swap up/down"
Pulling the stick against you will generate `move-up' events.
.sp
.\" .IP "0x20 WEIRD / enable double-click"
.\" If the press-to-click feature is enabled (see \fB\-b\fP), this
.\" will apparently generate \fItwo\fP clicks: one when pressing the
.\" stick down and another when releasing it. \fIUse with caution\fP:
.\" especially if `press-to-click' is not enabled, this is likely to
.\" mess up your mouse (issue an APM suspend/resume cycle to reset the
.\" trackpoint - but don't forget to kill
.\" .sp
.\" .B tp4d
.\" before doing that - it's designed to restore your settings
.\" after resuming the machine...).
.IP "0x20 invert z-axis"
DONT use this setting - it's likely to mess up the trackpoint.
(Note that the trackpoint keyboard has this enabled - obviously, it's
hardware wiring is different...)
.sp
.IP "0x40 swap x/y"
Pulling the stick will move the pointer left, pressing it to the
left will move the pointer down.
.sp
.IP "0x80 die"
.B tp4d
will not permit you to set this as it seems to lock up things.
In fact, this bit does `hard disable' the trackpoint device.
The only way to re-enable it is a power cycle (e.g. APM suspend).
.sp
.RE
.IP "\fB\-B"
Enable pointer motion backup when dragging.
.IP "\fB\-b"
Disable pointer motion backup when dragging. When clicking
the stick, the TrackPoint hardware discards the last motion events
received before detecting the release. This results in the
button-down and button-up events being generated at the same location.
However, when performing a drag, backing up the pointer motion may
be disturbing. This option disables the pointer motion backup
when dragging. (Pointer still backs up after a click.)
.IP "\fB\-2"
Set bits in the second mode word. Currently, only 0x8 - the "disable
pointer backup when dragging" feature is supported.
.SS Parameters
I found that there are a number of parameters that can be changed
an which will affect the trackpoint behavior.
.IP "\fB\-S\fR\fI x_y_sens\fR [0x00..0xff]"
Set the sensitivity of the trackpoint to pressure in
x-y-direction. 0xff yields maximal sensitivity, with this set to
0x00 not even an elephant will be capable to move the pointer on
the screen. (Works also if z-axis is disabled.)
.sp
.IP "\fB\-V\fR\fI upper_plateau_velocity\fR [0x00..0xff]"
Also changes the x-y behavior although it's difficult to
describe... Give it a try (works also if z-axis is disabled).
.sp
.IP "\fB\-P\fR\fI press_sens\fR [0x00..0xff]"
The mapping of z-axis pressure to button-press/release events
seems to detect pressure changes rather than absolute pressure
levels (although this plays a role, too). The \fIpress_sens\fP
parameter determines how fast a downwards oriented pressure change
must be to be recognized as a `button press'. E.g. when setting
this to 0x25, a much faster `push' is necessary than for a value
of 0x0a - the latter being a reasonable value. A very high value
will result in no button-press being detectid during the
down-pressing phase at all.
.sp
.IP "\fB\-R\fR\fI release_sens\fR [0x00..0xff]"
If no `button-press' had been detected during the down-pressing phase
(too small pressure change for the current value of
 \fIpress_sens\fP), this parameter determines the
`pressure-release' (or `upwards') acceleration that will result
in a \fIbutton-press\fP detection (right, during the
`release'-phase). Note that the corresponding button-release will
happen after the absolute pressure level reaching a certain
minimum, i.e. the result of the described scenario will be
a `button-click' when releasing the stick.
.sp
.IP "\fB\-T\fR\fI z-axis_averaging_timeconstant\fR [0x00..0xff]"
Give it a try - it's hard to describe...
.sp
.\" THE -U PARAMETER SUPPORT HAS BEEN DISABLED in release 2.0
.\"  .IP "\fB\-U\fR\fI backup_range\fR [0x00..0xff]"
.\"  Couldn't figure out the benefits of this parameter. Rather it seems
.\"  that changing it messes up the device. Better leave it alone...
.\"  .sp
.IP "\fB\-m\fR\fI button_mode\fR [0x00..0x0f]"
The \fIbutton_mode\fP setting determines which buttons will be
`simulated' by z-axis activity. All of the three mouse buttons
have an associated bit in this parameter. If this bit is set, the
corresponding button press/release is mimicked by z-axis activity.
E.g. \fB\-m\fP 0x02 will map the z-axis to the middle button.
Note that more than one bit may be set in the mask. The bit masks
0x01, 0x02, 0x04 correspond to the mouse buttons 1,2 and 3,
respectively.
.sp
There is another bit, 0x08, in this mode word that enables the
`press-to-click' feature, i.e. a button press (without release)
will automatically be converted to a single `click'.
.sp
.IP "\fB\-D\fR\fI minimum_drag\fR [0x00..0xff]"
This setting determines how far the down-pressed pointer must
be moved in order for a `drag' being detected.
.sp
.IP "\fB\-H\fR\fI drag_hysteresis\fR [0x00..0xfe]"
while `dragging', the force on the stick may be decreased. The end
of the `drag' will only be detected once the force reaches the
hysteresis value (a lower value means that you may drag with very
little force). Note that only even numbers are accepted, i.e. the
LSB is forced to 0.
.IP "\fB\-J\fR\fI Jenks_curvature\fR [0x00..0xff]"
I didn't reveal the effects of this yet - feedback welcome :-).
The IBM document explains this parameter, but it doesn't seem
to have a major impact.
.sp
.IP "\fB\-c\fR"
Force a calibration cycle: Tell the trackpoint that it should
recalibrate itself. Useful if the pointer is drifting.
.sp
.IP "\fB\-p\fR\fIpriority\fR"
Change the real-time scheduling priority of
.BR tp4d .
Note that this is \fInot the same\fP as the
.BR nice (1)
priority. Consult the \fBreal-time issues\fP section below for more
information. Note that this option is special; no white space must
be present between the option character and its argument.
.SH "REAL-TIME ISSUES"
Skip reading this section if you are not interested in
implementation details.
.SS "Race Condition Problems"
.B tp4d
was designed to work without having to patch the kernel.
Unfortunately,
the LINUX' generic pc_keybord/psaux driver provides no means for
`hooking' in a kind of `mouse support' module. (Nowadays there are
plenty of devices like the trackpoint, e.g. touchpads - most of
them are capable of doing better than a simple mouse. IMHO, the
way to go would be adding some hooks to the psaux driver which
would permit developing modules for all of these nice devices.
Hm - well, at present this is not possible without patching the
kernel, like I said).
.PP
The problem is that
.B tp4d
normally is competing with another process (most likely the
X-server or gpm) for the psaux device - the `other' process, which I
shall call X-server for simplicity, knows nothing about the
trackpoint extra features and is only interested in button and
motion events.
.B tp4d
on the other hand is not interested in these events but needs to
talk to the trackpoint for setting/reading some parameters. This means
that we now have \fItwo\fP processes reading from (and to a lesser
extent, writing to) the same device - race conditions are almost
inavoidable. Note that although it would be possible to patch the
X-server (e.g. adding a trackpoint extension), IMHO that is
not the way to go because it would restrict the trackpoint
support to a single application, the X-server.
.B tp4d
on the other hand works with \fIany\fP application that uses the
mouse.
.SS "Solving the Race Condition Problem"
The problem is solved by giving
.B tp4d
a higher, \fIreal-time\fP, scheduling priority
(\fBsched_setscheduler\fP(2)) than to the other process
(X-server). This guarantees that whenever
.B tp4d
is waiting for some data at the mouse port it will get scheduled
\fIbefore\fP any other process and hence will get the data it
needs. Note that this makes it necessary to lock a small amount of
memory (\fBmlock\fP(2)), about 2k or so, in order for having
crucial text and stack data always around (make sure the crucial
pieces of code never must wait for anything important getting
paged in).
.PP
While this scheme makes it possible to solve synchronization
of both, the X-server and
.B tp4d
reading psaux data, \fIwrite\fP access to the mouse must also
be synchronized (in case of the X-server by XFree, this issue
may more or less safely be ignored - the XFree server only
writes to the mouse when switching virtual consoles. This could
mess up things if switching virtual consoles
while 
.B tp4d
is handling an APM resume event or a trackpoint configuration request).
.PP
(Because I'm paranoid,) I also created a kernel module called `psctrl' that
supports a `psctrl' device which, while open, intercepts all
writes to /dev/psaux and, instead of writing the data out to
the hardware, forwards it to the /dev/psctrl device where it
can be picked up by
.BR tp4d .
Incoming data still is getting directly to the
psaux device itself - hence the real-time priority stuff
described above still applies.
.PP
Note that it would be much easier to simply `pipeline'
.B tp4d
\fIbetween\fP the X-server and the psaux device. However,
this would add the overhead of an extra context-switch for
every mouse event. (For the `write's this is not an issue since 
they can be assumed to only happen occasionally.)
.SS "Watchdog"
A process with real-time priority is inherently dangerous. If
ever, for some unknown reason, it should get into an endless loop,
only another process with the same or higher priority may kill it.
Usually, there is no shell with the necessary priority around and
even if there should be such a shell, most likely it doesnt have
the keyboard focus when things lock up (and the X-server never gets
scheduled).
.PP
For this reason, 
.B tp4d
has a built in watchdog: since
.B tp4d
is not very CPU intensive, its CPU limit is restricted to a small
amount e.g. 1 second. Every 20 seconds, the limit is re-adjusted
to 1 second ahead ( 1s plus the time already used). If ever the
process should hog the CPU (i.e. it's utilization reaches 5%) it
will be killed by the system.
.SH "SECURITY"
In order for being able to aquire the necessary resources,
.B tp4d
must be run SUID. However, immediately after getting the necessary
resources (the memory locks and the scheduling priority), the
extra privileges are dropped.
.SH "PIPES"
.B tp4d
communicates with clients through a pair of named pipes,
\fI/tmp/tp4pipe\fP for receiving client requests and
\fI/tmp/tp4epip\fP for sending replies. The protocol is quite
simple: most of the commands consist of one or two bytes. The
first byte, the \fIcommand code\fP is equal to the corresponding
command line option character which sets the desired parameter.
The parameter value itself is sent as a 1-byte binary value.
E.g. it is possible to enable the z-axis by issuing
.sp
echo -n 'M\\001' > /tmp/tp4pipe
.sp
or
.sp
echo -n 'Z' > /tmp/tp4pipe
.sp
The standard reply consists of a single character 'o' in case of
success and 'e' in case of an error.
.PP
Additional commands are:
.RS
.IP "'a' (no parameter)"
dump all cached settings to the hardware.
.IP "'?' (no parameter)"
query all cached settings; in this case, the reply consists of a
newline terminated sequence of characters which describes the
current settins in command line format, i.e. the reply may e.g. be used
to restart
.BR tp4d .
.PP
Note that only one single client should talk to 
.BR tp4d ;
use the \fBflock\fP(2) semantics for access synchronization to
\fB/tmp/tp4pipe\fP.
.SH "SIGNALS"
.B tp4d
catches the TERM, INT and QUIT signals, cleans up and exits.
Especially, the pipes are removed since their presence is 
used by
.B tp4d 
to determine if another instance is running already.
Upon reception of the HUP signal,
.B tp4d
will try to copy the ownership and mode bits of the
\fB/dev/psaux\fP device to the \fB/tmp/tp4pipe\fP and \fB/tmp/tp4epip\fP.
This can be
used by session/login scripts to reassign ownership to the pipes
after changing the mouse ownership. Note that this is likely to
fail if
.B tp4d
is not run by the superuser (remember that suid privileges are
dropped after startup).
.SH FILES
.IP "\fB/dev/psaux\fR the mouse device"
.IP "\fB/dev/psctrl\fR (optional) used to synchronize write access to the psaux device"
.IP "\fB/tmp/tp4pipe\fR request pipe (clients use \fBflock\fP(2))"
.IP "\fB/tmp/tp4epip\fR reply pipe"
.IP "\fBpsctrl.o\fR (optional) kernel module used to drive psctrl"
.SH SEE ALSO
.IP "\fBIBM trackpoint documentation\fP"
.IP "\fBThe source code\fP"
.IP "\fBsched_setscheduler\fP(2)"
.IP "\fBmlock\fP(2)"
.IP "\fBflock\fP(2)"
.IP "\fBxtp4\fP(1) the client for \fBtp4d\fP under X"
.SH "BUGS AND LIMITATIONS"
Device and pipe names are currently hardcoded. The real-time stuff
is inherently complex; some kernel support would be nice to make
things easier. At first, I had everything in a kernel patch, but
then - it's not easy to resume from APM in driver code, since there's
no usable process context available. A user mode daemon is nicer for
doing that. Also, a user mode daemon is (almost) never affected by
kernel version changes.
.PP
Raw z-axis data may be obtained using the burst mode described
by their docs. However, raw z-axis data is useless for normal 
applications (remember, 
.B tp4d
was developed, so `normal' apps can continue to use the trackpoint
like an ordinary PS2 mouse) - specialized applications have
to implement their own burst-mode protocol handler anyway...
.\"   .PP
.\"   Sorry, I have not discovered yet how to get raw z-axis data out of
.\"   the device.
.\" I now learned that there is detailed information about the trackpoint internals
.\" available on the web.
.\" Studying that info might allow for making the parameter
.\" naming more meaningfull and adding more features.
.PP
I dont really know why I put this silly digit `4' into the name of everything
- now I'm too lazy to rename all the stuff; just read tp4[l] (trackpoint for linux ...).
.PP
Most of the settings dealing with external pointing devices as well as
calibration parameter settings are not yet supported.
.SH "AUTHOR"
Till Straumann, <Till.Straumann|at|TU-Berlin.de>, 2000.
(replace |at| by @).
